/* cdu_types.h
 *
 * This file defines client data usage types
 *
 * Author: Cradlepoint Technology, Inc.  <source@cradlepoint.com>
 *		Adrian Sitterle <asitterle@cradlepoint.com>
 *
 * Copyright (C) 2019 Cradlepoint Technology, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#pragma once

#include <linux/list.h>
/* backwards compatible - remote code does not expect cumulative entries, but
 * instead expects only new data since last cleared
 * also see kernel nf_conn_counter counter: include/net/netfilter/nf_conntrack_acct.h
 */
#define ZERO_EPHEM_FLOWS

#undef DEBUG_ENABLE

#ifdef DEBUG_ENABLE
#define CDU_DEBUG(s, ...) pr_info_ratelimited("%s[%d]:" s, __func__, __LINE__, ##__VA_ARGS__)
#else
#define CDU_DEBUG(s, ...)
#endif

#define CDU_INFO(s, ...) pr_info_ratelimited("client data usage: " s, ##__VA_ARGS__)
#define CDU_ERROR(s, ...) pr_err_ratelimited("client data usage: " s, ##__VA_ARGS__)

#ifdef CONFIG_X86_64
#define CDU_USAGE_MAXENTRY 4096
#define USAGE_HASH_BITS		128
#else
#define CDU_USAGE_MAXENTRY 2048
#define USAGE_HASH_BITS		32
#endif

#define CDU_USAGE_DROP_CHUNK	((int) (CDU_USAGE_MAXENTRY / 10)) /* Drop 10 percent of the least recently seen clients */
#define IP_ALEN 4
#define IP6_ALEN 16

struct usage_stats {
	u_int64_t bytes;
	u_int64_t packets;
};

/* two datasets: ephemeral for current flows and persistent for finished flows */
#define STATSET(persist) ((persist) ? 0 : 1)

struct usage_entry {
	struct hlist_node node;
	struct list_head lnode;
	unsigned int key;		/* hash table key */

	union nf_inet_addr addr;	/* client IP addr */
	unsigned char family;		/* IP proptocol version */
	unsigned char mac[ETH_ALEN];	/* client MAC addr */

#ifdef ZERO_EPHEM_FLOWS
#define CLEARED_COUNTER 2
	struct usage_stats upload[3];	/* 0: persistent set, 1: ephemeral set, 2: cleared counts */
	struct usage_stats download[3];
#else
	struct usage_stats upload[2];	/* 0: persistent set, 1: ephemeral set */
	struct usage_stats download[2];
#endif

	unsigned long first_time;
	unsigned long last_time;
};

/* because struct xt_usage_tginfo in xt_usage.h */
#define usage_htable xt_usage_htable
struct xt_usage_htable {
	u_int8_t reset;
	u_int8_t usecount;

	struct proc_dir_entry *pde;
	char *name;
};
